"""Tools to run on PyPi distribution of packages."""
from string import Formatter

dflt_formatter = Formatter()


def increment_version(version_str):
    version_nums = list(map(int, version_str.split('.')))
    version_nums[-1] += 1
    return '.'.join(map(str, version_nums))


import requests
import json
from packaging.version import parse

URL_PATTERN = 'https://pypi.python.org/pypi/{package}/json'


# TODO: Use yb or merge functionality there if missing for json_package_info and
#  derivatives.
def json_package_info(package, url_pattern=URL_PATTERN):
    """Return version of package on pypi.python.org using json.

    >>> d = json_package_info('ps')
    >>> isinstance(d, dict)
    True
    >>> {'name', 'author', 'version'}.issubset(d['info'])
    True

    """
    response = requests.get(url_pattern.format(package=package))
    if response.status_code == requests.codes.ok:
        return response.json()
    else:
        return dict()


package_info = json_package_info


def _is_package_info_dict(package_info):
    # TODO: make it stronger, verifying the presence of required fields
    return isinstance(package_info, dict)


def ensure_json_package_info(package_info):
    """
    Gets and validates package info dicts from various sources.

    Namely, this is used in functions like ``releases``, ``release_dates`` etc. so
    that we can use them bother as fetcher+parser and as  parsers (giving them the
    same info dict we already fetched, so we don't have to fetch the info twice).
    """
    if isinstance(package_info, str):
        package_name = package_info
        package_info = json_package_info(package_name)
    assert _is_package_info_dict(
        package_info
    ), f'Not a valid package_info_ dict: {package_info}'
    return package_info


def release_versions(package):
    d = ensure_json_package_info(package)
    return list(d)


def release_dates(package):
    d = ensure_json_package_info(package)
    return [dd[-1]['upload_time'] for dd in d['releases'].values()]


def last_release_date(package):
    return release_dates(package)[-1]


def get_version(package, url_pattern=URL_PATTERN):
    """Return version of package on pypi.python.org using json.

    >>> get_version('py2store')  # doctest: +SKIP
    '0.1.15'

    """
    req = requests.get(url_pattern.format(package=package))
    version = parse('0')
    if req.status_code == requests.codes.ok:
        j = json.loads(req.text)
        releases = j.get('releases', [])
        for release in releases:
            ver = parse(release)
            if not ver.is_prerelease:
                version = max(version, ver)
    return version.base_version


def my_setup(**setup_kwargs):
    from setuptools import setup
    import json

    print('Setup params -------------------------------------------------------')
    print(json.dumps(setup_kwargs, indent=2))
    print('--------------------------------------------------------------------')
    setup(**setup_kwargs)


def ujoin(*args):
    """Join strings with the url seperator (/).

    Note that will add a / where it's missing (as in between 'https://pypi.org' and 'project/'),
    and only use one if two consecutive tokens use respectively end and start with a /
    (as in 'project/' and '/pipoke/').

    >>> ujoin('https://pypi.org', 'project/', '/pipoke/')
    'https://pypi.org/project/pipoke/'

    Extremal cases
    >>> ujoin('https://pypi.org')
    'https://pypi.org'
    >>> ujoin('https://pypi.org/')
    'https://pypi.org/'
    >>> ujoin('')
    ''
    >>> ujoin()
    ''
    """
    if len(args) == 0 or len(args[0]) == 0:
        return ''
    return (
        (args[0][0] == '/') * '/'  # prepend slash if first arg starts with it
        + '/'.join(x[(x[0] == '/') : (len(x) - (x[-1] == '/'))] for x in args)
        + (args[-1][-1] == '/') * '/'
    )  # append slash if last arg ends with it



########### Partial and incremental formatting ##########################################
class PartialFormatter(Formatter):
    """A string formatter that won't complain if the fields are only partially formatted.
    But note that you will lose the spec part of your template (e.g. in {foo:1.2f},
    you'll loose the 1.2f if not foo is given -- but {foo} will remain).
    """

    def get_value(self, key, args, kwargs):
        try:
            return super().get_value(key, args, kwargs)
        except KeyError:
            return '{' + key + '}'

    def format_fields_set(self, s):
        return {x[1] for x in self.parse(s) if x[1]}


partial_formatter = PartialFormatter()


# TODO: For those who love algorithmic optimization, there's some wasted to cut out here below.


def _unformatted(d):
    for k, v in d.items():
        if isinstance(v, str) and len(partial_formatter.format_fields_set(v)) > 0:
            yield k


def _fields_to_format(d):
    for k, v in d.items():
        if isinstance(v, str):
            yield from partial_formatter.format_fields_set(v)


def format_str_vals_of_dict(d, *, max_formatting_loops=10, **kwargs):
    """
    Traverse the values of a dict and apply string formatting to them, recursively.

    :param d: The template dict containing both format strings to be populated/embodied
    and (possibly) values to do so.
    :param max_formatting_loops: The maximum number of loops.
    :param kwargs: Extra values to use to populate string formats
    :return: A (shallow) copy of the filled out (i.e. populated/embodied) dict

    >>> d = {
    ...     'filepath': '{root}/{file}.{ext}',
    ...     'ext': 'txt'
    ... }
    >>> format_str_vals_of_dict(d, root='ROOT', file='FILE')
    {'filepath': 'ROOT/FILE.txt', 'ext': 'txt'}

    Note that if the input mapping `d` and the kwargs have a conflict, the mapping
    version is used!

    >>> format_str_vals_of_dict(d, root='ROOT', file='FILE', ext='will_not_be_used')
    {'filepath': 'ROOT/FILE.txt', 'ext': 'txt'}

    But if you want to override an input mapping, you can -- the usual way:

    >>> format_str_vals_of_dict(dict(d, ext='will_be_used'), root='ROOT', file='FILE')
    {'filepath': 'ROOT/FILE.will_be_used', 'ext': 'will_be_used'}

    If you don't provide enough fields to satisfy all the format fields in the values
    of ``d``, you'll be told to bugger off.

    >>> format_str_vals_of_dict(d, root='ROOT')
    Traceback (most recent call last):
    ...
    ValueError: I won't be able to complete that. You'll need to provide the values for:
      file

    And it's recursive...

    >>> d = {
    ...     'filepath': '{root}/{filename}',
    ...     'filename': '{file}.{ext}'
    ... }
    >>> my_configs = {'root': 'ROOT', 'file': 'FILE', 'ext': 'EXT'}
    >>> format_str_vals_of_dict(d, **my_configs)
    {'filepath': 'ROOT/FILE.EXT', 'filename': 'FILE.EXT'}

    # TODO: Could make the above work if filename is give, but not file nor ext!
    #  At least as an option.

    """
    d = dict(**d)  # make a shallow copy
    # The defaults (kwargs) cannot overlap with any keys of d, so:
    kwargs = {k: kwargs[k] for k in set(kwargs) - set(d)}

    provided_fields = set(d) | set(kwargs)
    missing_fields = set(_fields_to_format(d)) - provided_fields

    if missing_fields:
        raise ValueError(
            "I won't be able to complete that. You'll need to provide the values for:\n"
            + f"  {', '.join(missing_fields)}"
        )

    for i in range(max_formatting_loops):
        unformatted = set(_unformatted(d))

        if unformatted:
            for k in unformatted:
                d[k] = partial_formatter.format(d[k], **kwargs, **d)
        else:
            break
    else:
        raise ValueError(
            f'There are still some unformatted fields, '
            f'but I reached my max {max_formatting_loops} allowed loops. '
            + f'Those fields are: {set(_fields_to_format(d)) - (set(d) | set(kwargs))}'
        )

    return d



import inspect
import re
import pickle
import pkgutil
import builtins
import os

from pipoke import dpath
from pipoke.pypi_store import pkg_name_stub

standard_lib_dir = os.path.dirname(os.__file__)


def is_standard_lib_path(path):
    return path.startswith(standard_lib_dir)


def standard_lib_module_names(is_standard_lib_path=is_standard_lib_path,
                              name_filt=lambda name: not name.startswith('_')):
    return filter(name_filt, (module_info.name for module_info in pkgutil.iter_modules()
                              if is_standard_lib_path(module_info.module_finder.path)))


local_pkg_names = {x.name for x in pkgutil.iter_modules() if x.ispkg}

builtin_pkg_names = set(standard_lib_module_names())
builtin_obj_names = {x.lower() for x in dir(builtins)}

py_reserved_words = {
    'and', 'as', 'assert', 'break', 'class', 'continue', 'def', 'del', 'elif', 'else',
    'except', 'exec', 'finally',
    'for', 'from', 'global', 'if', 'import', 'in', 'is', 'lambda', 'not', 'or', 'pass',
    'print', 'raise', 'return',
    'try', 'while', 'with', 'yield'
}

builtin_names = builtin_pkg_names | builtin_obj_names | py_reserved_words


def get_all_words_from_wordnet():
    from nltk.corpus import wordnet as wn
    return set(wn.all_lemma_names())


try:
    all_words = pickle.load(open(dpath('all_words.p'), 'rb'))
except:
    all_words = get_all_words_from_wordnet()

simple_words = set(filter(re.compile('[a-z]+$').match, all_words))

pypi_pkg_names = set(pkg_name_stub)
pkg_names = pypi_pkg_names.union(builtin_names)  # pypi + builtin pkg names
pkg_names_that_are_words = all_words.intersection(pkg_names)

available_simple_words = simple_words - pkg_names


def str_for_func(func):
    s = func.__name__
    if s != '<lambda>':
        return s
    else:
        fs = inspect.getsource(func)
        fs = fs[:fs.index('\n')]
        lambda_re = re.compile('.+lambda (.+)')
        return lambda_re.match(fs).group(1)


def disp_str_for_counts(words, pkgs, pkgs_words):
    s = ''
    s += f"{len(words)} words"
    s += f"{len(pkgs)} package names"
    s += f"{len(pkgs_words)} that are both"
    return s


def words_and_pkg_names_satisfying_condition(cond, print_counts=False, cond_str=None):
    words = list(filter(cond, all_words))
    pkgs = list(filter(cond, pkg_names))
    pkgs_words = list(filter(cond, pkg_names_that_are_words))
    if print_counts:
        if cond_str is None:
            cond_str = str_for_func(cond)
        print(f"{cond_str}")
        print(disp_str_for_counts(words, pkgs, pkgs_words))
    return words, pkgs, pkgs_words


def words_and_pkg_names_satisfying_regex(regex, print_counts=False):
    """
    Get (English) words, pypi package names, and strings that are both words and package names.
    :param regex: Regular expression that should match the WHOLE string
    :param print_counts: Print count statistics
    :return:
    """
    return words_and_pkg_names_satisfying_condition(re.compile(regex).match, print_counts=print_counts, cond_str=regex)


def is_not_a_pkg_name(regex='.*', words=None):
    """
    Filter words, keeping only those that are not already pypi package names.
    :param regex: Extra (regular expression) filter to apply
    :param words: The words you want to filter. By default will take all (simple) words of the (wordnet) dictionary.
    :return: The subset of input words that are not pypi package names
    """
    words = words or simple_words
    if isinstance(words, str):
        words = set(map(lambda w: w.strip(), words.split(',')))
    return set(filter(re.compile(regex).match, words)).difference(pkg_names)


if __name__ == '__main__':
    import argh

    parser = argh.ArghParser()


    def allwords():
        return all_words


    def pkgnames():
        return pkg_names


    parser.add_commands([words_and_pkg_names_satisfying_regex, is_not_a_pkg_name, allwords, pkgnames])
    parser.dispatch()



"""Pip installed package diagnosis
The tools here will help you install and diagnose packages in a test virtual 
environment.
"""

import os
import sys
import subprocess
import pkg_resources
import json
from typing import Union, Iterable


def current_virtual_environment_path():
    """
    Returns the path of the current virtual environment the function is called from
    """
    return os.environ.get('VIRTUAL_ENV', None)


DFLT_TEST_ENV = current_virtual_environment_path()


def venv_virtualenv_exists(virtual_env):
    return os.path.isdir(virtual_env)


def fullpath(path):
    return os.path.abspath(os.path.expanduser(path))


def virtualenv_path(virtual_env, virtual_envs_dir=''):
    """
    Get the path of a virtual environment given its name.
    """
    return fullpath(os.path.join(virtual_envs_dir or '', virtual_env))


def create_virtualenv(
    virtual_env=DFLT_TEST_ENV,
    virtual_envs_dir='',
):
    """
    Create a virtual environment using specified managers if it doesn't exist.

    :param virtual_env: The path to the virtual environment
    :return True if the virtual environment exists or was created successfully, False otherwise
    """
    virtual_env = virtualenv_path(virtual_env, virtual_envs_dir=virtual_envs_dir)

    if venv_virtualenv_exists(virtual_env):
        print(f"INFO: Virtual environment already exists: {virtual_env}")
    else:
        print(f"INFO: Creating virtual environment: {virtual_env}")
        subprocess.run([sys.executable, '-m', 'venv', virtual_env], check=True)

    return virtual_env  # Exit the function if venv succeeded


def activate_virtualenv(virtualenv_path):
    """
    Activate a virtual environment given its path.
    """
    # Determine the current shell
    shell = os.path.basename(os.getenv('SHELL'))

    if shell in ('bash', 'zsh'):
        activation_script = 'activate'
    elif shell == 'fish':
        activation_script = 'activate.fish'
    elif sys.platform == 'win32':
        activation_script = 'Scripts/activate.bat'
    else:
        raise NotImplementedError("Unsupported shell or platform")

    activation_script_path = os.path.join(virtualenv_path, activation_script)

    # Check if the activation script exists
    if not os.path.exists(activation_script_path):
        raise FileNotFoundError(
            f"Activation script not found: {activation_script_path}"
        )

    # Activate the virtual environment
    try:
        subprocess.run(f"source {activation_script_path}", shell=True)
    except subprocess.CalledProcessError:
        raise RuntimeError("Failed to activate the virtual environment")


def install_package(pkg_name, virtual_env=DFLT_TEST_ENV):
    """
    Install a package in the virtual environment if not already installed.
    """
    try:
        subprocess.run(
            [os.path.join(virtual_env, 'bin', 'pip'), 'install', pkg_name], check=True
        )
    except subprocess.CalledProcessError:
        pass


def is_package_installed(pkg_name):
    """
    Check if a package is installed in the virtual environment.
    """
    try:
        pkg_resources.get_distribution(pkg_name)
        return True
    except pkg_resources.DistributionNotFound:
        return False


def dflt_pkg_diagnosis(imported_pkg):
    non_dunder_attributes = [
        attr for attr in dir(imported_pkg) if not attr.startswith('__')
    ]
    return {
        'non_dunder_attributes_count': len(non_dunder_attributes),
    }


def run_pkg_diagnosis(pkg_name, pkg_diagnosis=dflt_pkg_diagnosis):
    """
    Run a diagnosis function on the imported package.
    """
    try:
        imported_pkg = __import__(pkg_name)
        return pkg_diagnosis(imported_pkg)
    except ImportError:
        return None


def dflt_folder_diagnosis(pkg_folder):
    total_files = sum(len(files) for _, _, files in os.walk(pkg_folder))
    total_bytes = sum(
        os.path.getsize(os.path.join(root, file))
        for root, _, files in os.walk(pkg_folder)
        for file in files
    )

    return {
        'total_files': total_files,
        'total_bytes': total_bytes,
        # Add other useful information here
    }


def run_folder_diagnosis(pkg_name, folder_diagnosis=dflt_folder_diagnosis):
    """
    Run a diagnosis function on the folder containing the package's code folder.
    """
    try:
        pkg_info = pkg_resources.get_distribution(pkg_name)
        pkg_location = pkg_info.location
        pkg_folder = os.path.join(pkg_location, pkg_name.replace('-', '_'))

        return folder_diagnosis(pkg_folder)
    except pkg_resources.DistributionNotFound:
        print(f"ERROR: Package not found: {pkg_name}")
        return None


import subprocess


def run_pkg_tests(pkg_name, virtual_env=DFLT_TEST_ENV):
    """
    Run tests (including doctests, unittest tests, and pytest tests) for the installed package.
    """
    import doctest
    import unittest

    try:
        pkg_info = pkg_resources.get_distribution(pkg_name)
        pkg_location = pkg_info.location
        pkg_folder = os.path.join(pkg_location, pkg_name.replace('-', '_'))

        test_diagnoses = {}

        # Run doctests using doctest module
        try:
            doctest_results = doctest.testmod(pkg_name)
            test_diagnoses['doctest_results'] = {
                'attempts': doctest_results.attempted,
                'failures': doctest_results.failed,
            }
        except Exception:
            pass

        # Run unittest tests
        loader = unittest.TestLoader()
        suite = loader.discover(pkg_folder, pattern="test_*.py")
        unittest_runner = unittest.TextTestRunner(verbosity=2)
        unittest_result = unittest_runner.run(suite)
        test_diagnoses['unittest_results'] = {
            'total_tests_found': unittest_result.testsRun,
            'total_failures': len(unittest_result.failures),
            'total_errors': len(unittest_result.errors),
        }

        try:
            # Run pytest tests within the virtual environment
            cmd = [
                os.path.join(virtual_env, 'bin', 'python'),
                '-m',
                'pytest',
                pkg_folder,
            ]
            # Capture the output for analysis if needed
            result = subprocess.run(
                cmd,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                universal_newlines=True,
            )
            # Analyze the pytest output, parse the results, and extract relevant information
            # Example: You can count the number of passed and failed tests from the output
            # Sample code to count passed and failed tests (modify as needed)
            passed_tests = result.stdout.count("PASSED")
            failed_tests = result.stdout.count("FAILED")

            # Get test diagnoses
            test_diagnoses['pytest_results'] = {
                'total_tests_found': passed_tests + failed_tests,
                'total_passed_tests': passed_tests,
                'total_failed_tests': failed_tests,
            }
        except subprocess.CalledProcessError or Exception:
            pass

        return test_diagnoses

    except pkg_resources.DistributionNotFound:
        return None


def diagnose_pkgs(pkgs: Iterable[str]):
    if isinstance(pkgs, str):
        pkgs = pkgs.split()
    d = {}
    for pkg in pkgs:
        d[pkg] = diagnose_pkg(pkg)
    return d


def dflt_json_info_extractor(pkg_name):
    from pipoke.distribution import package_info
    from dol import path_get

    info_dict = package_info(pkg_name)
    extractor = path_get.paths_getter(
        ['info.description', 'info.summary', 'info.author', 'info.version', 'releases'],
        on_error = path_get.return_none_on_error,
    )
    d = extractor(info_dict)
    releases = d.pop('releases')
    if releases:
        release = next(iter(releases[d['info.version']]), None)
        if release:
            d['release.size'] = release['size']
            d['relase.upload_time_iso_8601'] = release['upload_time_iso_8601']
    return d


def diagnose_pkg(
    pkg_name: str,
    # virtual_env: str = DFLT_TEST_ENV,
    # *,
    # virtual_envs_dir: str = '',
    # delete_created_virtual_env_when_done: bool = True,
):
    # Initialize the results with default values
    pkg_was_in_environment = False

    # virtual_env = virtualenv_path(virtual_env, virtual_envs_dir=virtual_envs_dir)
    # virtual_env_existed = venv_virtualenv_exists(virtual_env)

    # # Create the virtual environment if it doesn't exist
    # virtual_env = create_virtualenv(virtual_env)

    # Check if the package is already in the environment
    pkg_was_in_environment = is_package_installed(pkg_name)

    # Install the package if not in the environment
    if not pkg_was_in_environment:
        install_package(pkg_name)

    d = dict()

    try:
        d['pkg_diagnosis_result'] = run_pkg_diagnosis(pkg_name)
    except Exception as e:
        print(f"ERROR: pkg_diagnosis_result: {e}")

    try:
        d['pkg_folder_diagnosis_result'] = run_folder_diagnosis(pkg_name)
    except Exception as e:
        print(f"ERROR: pkg_folder_diagnosis_result: {e}")

    try:
        d['test_diagnosis_result'] = run_pkg_tests(pkg_name)
    except Exception as e:
        print(f"ERROR: test_diagnosis_result: {e}")

    try:
        d['json_info'] = dflt_json_info_extractor(pkg_name)
    except Exception as e:
        print(f"ERROR: json_info: {e}")

    # Uninstall the package if it was installed in this process
    if not pkg_was_in_environment:
        subprocess.run(['pip', 'uninstall', '-y', pkg_name])

    # if delete_created_virtual_env_when_done and not virtual_env_existed:
    #     # Delete the virtual environment if it was created in this process
    #     print(f"DONE: Deleting virtual environment: {virtual_env}")
    #     subprocess.run(['rm', '-rf', virtual_env])

    return d


# def _run_diagnoses(pkg_name):
#     d = {
#         # 'pkg_was_in_environment': pkg_was_in_environment,
#         'pkg_diagnosis_result': run_pkg_diagnosis(pkg_name),
#         'pkg_folder_diagnosis_result': run_folder_diagnosis(pkg_name),
#         'test_diagnosis_result': run_pkg_tests(pkg_name),
#     }

#     return json.dumps(d, indent=2)


# def run_script_in_virtualenv(script_path, virtual_env, *args):
#     python_executable = os.path.join(virtual_env, 'bin', 'python')
#     cmd = [python_executable, script_path] + list(args)
#     result = subprocess.run(
#         cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True
#     )
#     return result.stdout


# def run_pkg_diagnosis(pkg_name, virtual_env=DFLT_TEST_ENV):
#     # path to your standalone script for package diagnosis
#     script_path = 'path/to/pkg_diagnosis_script.py'
#     return run_script_in_virtualenv(script_path, virtual_env, pkg_name)


# Extras for pyenv:


def virtualenv_manager(
    virtual_env: str,
    virtual_envs_dir: str = '',  # empty string will have the effect of looking in the current directory
) -> Union[str, None]:
    """
    Check if a virtual environment was created using a specific manager,
    returning the name of the manager if found.

    :param virtual_env: The name of the virtual environment
    :param virtual_envs_dir: The directory where virtual environments are stored
    :return: The name of the virtual environment manager if found, None otherwise

    """
    virtual_envs_dir = virtual_envs_dir or ''
    virtual_env_path = os.path.join(virtual_envs_dir, virtual_env)
    # Check if it's a venv-created virtual environment
    venv_marker = os.path.join(virtual_env_path, 'pyvenv.cfg')
    if os.path.exists(venv_marker):
        return 'venv'

    # Check if it's a pyenv-created virtual environment
    pyenv_version_file = os.path.join(virtual_env_path, '.python-version')
    if os.path.exists(pyenv_version_file):
        return 'pyenv'

    return None


def pyenv_exists():
    """
    Check if pyenv is installed.
    """
    try:
        subprocess.run(['pyenv', '--version'], check=True, capture_output=True)
        return True
    except FileNotFoundError:
        return False


def create_virtualenv_with_pyenv(virtual_environment_name):
    """
    Create a virtual environment using pyenv if it doesn't exist.

    :param virtual_environment_name: The name of the virtual environment
    :return True if the virtual environment exists or was created successfully, False otherwise
    """
    # Check if pyenv is installed
    if not pyenv_exists():
        raise ValueError("ERROR: pyenv not found")

    # Check if the virtual environment exists
    virtualenv_path = get_pyenv_virtualenv_path(virtual_environment_name)
    if virtualenv_path:
        print(f"INFO: Virtual environment already exists: {virtualenv_path}")
        return virtualenv_path

    # Create the virtual environment
    try:
        subprocess.run(['pyenv', 'virtualenv', virtual_environment_name], check=True)
        return get_pyenv_virtualenv_path(virtual_environment_name)
    except subprocess.CalledProcessError:
        return False


def activate_virtualenv_with_pyenv(virtual_environment_name):
    """
    Activate a virtual environment using pyenv given its name.
    """
    # Check if pyenv is installed
    if not pyenv_exists():
        raise ValueError("ERROR: pyenv not found")

    # Check if the virtual environment exists
    virtualenv_path = get_pyenv_virtualenv_path(virtual_environment_name)
    if not virtualenv_path:
        raise ValueError(
            f"ERROR: Virtual environment not found: {virtual_environment_name}"
        )

    # Activate the virtual environment
    try:
        subprocess.run(['pyenv', 'activate', virtual_environment_name], check=True)
    except subprocess.CalledProcessError:
        raise RuntimeError("Failed to activate the virtual environment")


def get_pyenv_virtualenv_path(virtual_environment_name):
    """
    Get the path of a virtual environment managed by pyenv given its name.
    """
    try:
        # Run the pyenv virtualenv-prefix command and capture the output
        cmd = ['pyenv', 'virtualenv-prefix', virtual_environment_name]
        result = subprocess.run(
            cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True
        )

        # Check if the command was successful and has output
        if result.returncode == 0 and result.stdout:
            return (
                result.stdout.strip()
            )  # Remove leading/trailing whitespace and return the path
        else:
            return None  # Return None if the command fails or has no output
    except FileNotFoundError:
        # Handle the case where pyenv is not found
        return None


# script to run diagnose_pkgs from the command line
if __name__ == '__main__':
    import argparse

    parser = argparse.ArgumentParser(
        description='Run package diagnosis on a list of packages'
    )
    parser.add_argument(
        'packages',
        metavar='package',
        type=str,
        nargs='+',
        help='The names of the packages to diagnose',
    )
    parser.add_argument(
        '--virtual-env',
        type=str,
        default=DFLT_TEST_ENV,
        help='The name of the virtual environment to use',
    )
    parser.add_argument(
        '--virtual-envs-dir',
        type=str,
        default='',
        help='The directory where virtual environments are stored',
    )
    parser.add_argument(
        '--delete-created-virtual-env-when-done',
        action='store_true',
        help='Delete the virtual environment when done',
    )
    args = parser.parse_args()

    # run diagnose_pkgs
    d = diagnose_pkgs(args.packages)

    # print the results
    print(json.dumps(d, indent=2))
